#include <allegro.h>
#ifdef WIN32
  #include <winalleg.h>  // BASS causes some conflicts, so this must be included
#endif
#include <alfont.h>
#include <bass.h>
#include <fstream>
#include "main.h"
#include "basic.h"
#include "units.h"
#include "map.h"
#include "items.h"
#include "globals.h"
#include "controls.h"
#include "special.h"

// global variables
BITMAP* Offscreen;
BITMAP* Mouse;
int gfx = 0;
static volatile int fps = 0;
static volatile int mytime = 0;
CMap* Map;
bool bPlaying = false;
ALFONT_FONT* MyFont;
CTableGame* RiddleTable = NULL;
CBookReader* BookReader = NULL;
static HMUSIC BackMusic;
static CMainMenu* StartupMenu = NULL;

inline void Fps()
{
    fps--;    
}  

inline void Timer()               
{
    mytime++;
}    

// returns value different from 0 if something goes wrong
inline int Initialization()
{
    // initialize random number generator
    srand(time(NULL));
    // initialize Allegro
    if (allegro_init() != 0) return 1;
    // load the INI file
    set_config_file("macro.ini");
    // set graphics
    set_color_depth(32);
    gfx = get_config_int("Globals","Windowed",1) == 0 ? GFX_AUTODETECT_FULLSCREEN : GFX_AUTODETECT_WINDOWED;
    if (set_gfx_mode(gfx,RESOLUTION_X,RESOLUTION_Y,0,0) != 0) return 2;
    // install keyboard, mouse, timer
    if (install_keyboard() != 0) return 3;    
    if (install_mouse() == -1) return 4;       
    if (install_int_ex(Fps, BPS_TO_TIMER(20)) != 0) return 5;    // force gameplay at 20 for all computers
    if (install_int_ex(Timer, BPS_TO_TIMER(1)) != 0) return 6;  // timer used for gameplay
    if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, 0) != 0) return 7;  // Allegro sound system
    // transparent text
    text_mode(-1);
    // create offscreen bitmap
    Offscreen = create_bitmap(SCREEN_W,SCREEN_H);
    // load mouse pointer
    Mouse = load_bitmap(get_config_string("Globals","Mouse","mouse.xx"),NULL);
    if (Mouse == NULL) return 8;
    // load the game map
    bool ok = true;
    Map = new CMap(get_config_string("Globals","Map","macromap.map"),ok);
    if (!ok) return 9;
    // load miscelaneous pictures
    if (!LoadItemPictures()) return 10;
    // load unit pictures
    if (!LoadUnitPictures()) return 11;
    for (int p=0; p<Map->iWidth; p++)
      for (int q=0; q<Map->iHeight; q++)
        if (Map->DataMap[p][q] == 1)   // player starting location
             Player = new CPlayer((p+1)*SQUARE_WIDTH,(q+1)*SQUARE_HEIGHT>>1,get_config_int("Player","HP",1000));   
    // load unit sounds
    if (!LoadUnitSounds()) return 12;  
    // initialize AlFont
    if (alfont_init() != 0) return 13;
    alfont_text_mode(-1);
    // load font type
    MyFont = alfont_load_font("greek.ttf");
    // create main menu
    Menu = new CMenu();  
    // create the enemy units
    CreateMonsters();            
    // load background music
    if (!BASS_Init(1, 44000, BASS_DEVICE_3D, 0, NULL)) return 14;
    BackMusic = BASS_MusicLoad(false, get_config_string("Globals","Backsound","magmatic.xm"), 0, 0, BASS_SAMPLE_LOOP,0);
    if (BackMusic == 0) return 15; 
    // play the music!
    BASS_ChannelPlay(BackMusic,true);
    // player hasn't read the book yet
    bBookRead = false;
    // we can start playing the game now!
    bPlaying = true;
    // set screen starting position
    Map->iGlobalX = 380;
    Map->iGlobalY = 2570;
    // everything's ok
    return 0;
}

inline void Deinitialization()
{
    if (StartupMenu != NULL) delete StartupMenu;
    if (BookReader != NULL) delete BookReader;
    BASS_Stop();
    BASS_Free();
    if (RiddleTable != NULL) delete RiddleTable;
    DestroyBullets();
    DestroyMonsters();
    alfont_destroy_font(MyFont);
    alfont_exit();
    delete Menu;
    delete Player;
    DestroyUnitSounds();
    DestroyUnitPictures();
    DestroyItemPictures();
    delete Map;
    destroy_bitmap(Mouse);
    destroy_bitmap(Offscreen);
    allegro_exit();
}     

int main()
{
    if (int e = Initialization() != 0)
    {
        system("Error while initializing the game!");    
        ofstream Debug("debug.txt");
        Debug << "Exited with error code: " << e << endl;
        Deinitialization();        
        return 1;
    }    
    Menu->iState = STARTUP_STATE;
    Player->szName = "Some name";
    int iEffectCounter = 1;  // MUST not be 0 !!!!  
    volatile int iStartTime = mytime;    
    StartupMenu = new CMainMenu(3);   
    SAMPLE *TmpSnd = NULL;     
    while (bPlaying)
    {                
        if (fps <= 0)
        {        
          fps = 1;         
          if (!bPlayMusic) 
            BASS_ChannelPause(BackMusic);
          else if ( (bPlayMusic) && (BASS_ChannelIsActive(BackMusic) == BASS_ACTIVE_PAUSED) )
            BASS_ChannelPlay(BackMusic, false);
          GarbageCollector(); 
          switch (Menu->iState)
          {
            case RIDDLE_STATE: clear_bitmap(Offscreen);
                               RiddleTable->XO();
                               if (RiddleTable->Victory)
                               {                                   
                                   delete RiddleTable;
                                   RiddleTable = NULL;                                   
                                   iStartTime = mytime;
                                   iEffectCounter = 1;
                                   Menu->iState = TRANSITION_STATE;
                               }    
                               break;
            case WINNER_STATE: if (mytime >= iStartTime + 15)
                               {
                                   bPlaying = false;
                               }
                               else
                               {    
                                  clear_bitmap(Offscreen);
                                  alfont_set_font_size(MyFont,48);
                                  alfont_textout(Offscreen,MyFont,"Victory!!!",160,20,makecol(55,100,100));
                                  alfont_set_font_size(MyFont,30);
                                  alfont_textout(Offscreen,MyFont,"And so the world was safe again...",50,100,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"If you liked this game",120,150,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"and want to see Episode 2",120,185,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"then mail me at",120,215,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"gcsaba2@freemail.hu",120,250,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"and maybe I'll write",120,285,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"it someday ;)",120,310,makecol(255,100,0));
                               }
                               break;
            case LOSER_STATE: if (mytime >= iStartTime + 5)
                               {
                                   bPlaying = false;
                               }
                               else
                               {    
                                  clear_bitmap(Offscreen);
                                  alfont_set_font_size(MyFont,48);
                                  alfont_textout(Offscreen,MyFont,"You lost :(",150,20,makecol(255,100,100));
                                  alfont_set_font_size(MyFont,30);
                                  alfont_textout(Offscreen,MyFont,"If you liked this game",120,100,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"and want to see Episode 2",120,135,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"then mail me at",120,170,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"gcsaba2@freemail.hu",120,205,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"and maybe I'll write",120,240,makecol(255,100,0));
                                  alfont_textout(Offscreen,MyFont,"it someday ;)",120,275,makecol(255,100,0));
                               }
                               break;
            case STARTUP_STATE: switch (StartupMenu->MenuResult(mytime))
                                {
                                    case 0: break;
                                    case 1: Menu->iState = NOISED_STATE;
                                            delete StartupMenu;
                                            StartupMenu = NULL;
                                            iStartTime = mytime;
                                            break;
                                    case -1: bPlaying = false;
                                             break;
                                }    
                                break;
            case NOISED_STATE: FallAppart(Offscreen,iEffectCounter);                                
                               if (iStartTime + 3 == mytime)
                                  Menu->iState = ATTACK_STATE;
                               break;
            case TRANSITION_STATE: Mosaic(Offscreen,iEffectCounter);                                                           
                                   if (mytime == iStartTime + 5)  // run for 5 seconds
                                   {
                                     Menu->iState = ATTACK_STATE;
                                   }    
                                   break;
            case READING_STATE: if (!BookReader->ReadTheBook(mytime))
                                {
                                    delete BookReader;
                                    BookReader = NULL;
                                    bBookRead = true;   // oh no!
                                    iStartTime = mytime;
                                    iEffectCounter = 1;
                                    Menu->iState = TRANSITION_STATE;
                                    Player->iState = STOP;
                                }    
                                break;
            default: clear_bitmap(Offscreen);
                     Map->DoDraw();
                     Map->DoScroll();
                     Map->CameraToCoord(Player->x-Map->iGlobalX,Player->y-Map->iGlobalY);                             
                     Player->DoAction();
                     DrawBullets();
                     MonsterAction();
                     BulletAction();        
                     Menu->DoDraw();
                     DrawTalking();
                     MouseActions();
                     //textprintf(Offscreen,font,10,80,255,"Player state = %d",Player->iState);
                     break;
          }    
        }    
        if ((Player->HP < 0) && (Menu->iState != LOSER_STATE) )
        {
            TmpSnd = load_sample(get_config_string("Globals","Boo","boo.wav"));
            play_sample(TmpSnd,SampleVolume,0,1000,0);
            Menu->iState = LOSER_STATE;
            iStartTime = mytime;            
        }    
        else if ( (Troll->HP < 0) && (Menu->iState != WINNER_STATE) )
        {
            TmpSnd = load_sample(get_config_string("Globals","Applause","applause.wav"));
            play_sample(TmpSnd,SampleVolume,0,1000,0);
            Menu->iState = WINNER_STATE;                
            iStartTime = mytime;
        }    
        draw_sprite(Offscreen,Mouse,mouse_x,mouse_y);
        blit(Offscreen,screen,0,0,0,0,SCREEN_W,SCREEN_H);                          
    }    
    destroy_sample(TmpSnd);
    Deinitialization();    
    return 0;
}
END_OF_MAIN();    
