/*
***************************************************
*                                                 *
*                    THE MAP                      *
*                                                 *
***************************************************

This is where the map class is implemented, including its 
drawing to the screen, and reading from a file.

Author: Gajo Csaba
Date: 25 Feb, 2005
Version: 0.2
*/

#ifndef _map_h
#define _map_h

#define SQUARE_WIDTH 64
#define SQUARE_HEIGHT 64
#define MAX_TILE 10    // number of different tiles
#define SQUARE_ELEVATION 13

#define MOUSEMAP_WIDTH SQUARE_WIDTH
#define MOUSEMAP_HEIGHT SQUARE_HEIGHT

#define SPECIAL_BOOK 3
#define SPECIAL_COMPUTER 8
#define SPECIAL_HEALTH 2

class BITMAP;

class CMap
{
    protected:
        // special locations
        int iBookX,iBookY;
        int iComputerX,iComputerY;
        int* iHealthX;
        int* iHealthY;
        int iHealthCount;
        BITMAP* MouseMap;
        inline void MouseMapping(int X, int Y, int &iTileX, int &iTileY);
    public:
        int** DataMap;   // matrix
        BITMAP* Tiles[MAX_TILE];
        int iWidth;
        int iHeight;
        volatile int iGlobalX;    // screen offset
        volatile int iGlobalY;                   
        CMap(const char* szFileName, bool &ok);
        ~CMap();
        void DoDraw();
        void DoScroll();
        void CameraToCoord(int x, int y);
        bool Collision(int &x, int &y, int w, int h);
        /*  Checks if the player has stepped on a special location
            RETURN VALUES:
                0 - no
                1 - book
                2 - computer
                3 - health
        */
        int SteppedOnSpecialLocation(int x, int y);
};    

extern int CameraSpeed;

#endif
