#include <allegro.h>
#include <alfont.h>
#include "special.h"
#include "main.h"
#include "globals.h"
#include "units.h"
#include "map.h"

#define EMPTY 0
#define X 1
#define O 2
#define OFFSET 35

// Picture = bitmap on which you do the effect
// iCounter = square size, increased by 1 in each step
void Mosaic(BITMAP* Pic, int &iCounter)
{
    int iPixel;
    for (int y=0; y<Pic->h; y += iCounter)
    {
        for (int x=0; x<Pic->w; x += iCounter)
        {
            iPixel = getpixel(Pic,x,y);   // get pixel at top-left
            for (int j=y; j<y+iCounter; j++)
            {
                for (int i=x; i<x+iCounter; i++)
                {
                    // put that pixel from (x,y) to (x+iCounter,y+iCounter)
                    putpixel(Pic,i,j,iPixel);  
                }    
            }    
        }
    }        
    iCounter++;  // increase the square size
}  

void FallAppart(BITMAP* Pic, int &iCounter)
{
    for (int y=iCounter; y<Pic->h; y++)
    {
        for (int x=iCounter; x<Pic->w; x++)
        {
            if ((getpixel(Pic,x,y) != makecol(128,255,255)) &&
                (getpixel(Pic,x,y) != makecol(255,255,128)) )
              putpixel(Pic,x,y,getpixel(Pic,rand() % Pic->w,rand() % Pic->h));
        }    
    } 
    iCounter++;
}    

// ----------------------- CTableGame ---------------------

CTableGame :: CTableGame()
{
    for (int i=0; i<3; i++)
      for (int j=0; j<3; j++)
        table[i][j] = EMPTY;
    Victory = false;
    bComputersTurn = true;
}
    
CTableGame :: ~CTableGame()
{
    // nothing to do..
}
    

void CTableGame :: DrawTable()
{
    int a = SCREEN_W >> 2;
    int b = SCREEN_H >> 2;
    int n = (SCREEN_W - (a<<1)) / 3;
    int m = (SCREEN_H - (b<<1)) / 3;
    rectfill(Offscreen,a,b,SCREEN_W - a, SCREEN_H - b,makecol(255,255,128));
    rect(Offscreen,a,b,SCREEN_W-a,SCREEN_H-b,makecol(255,0,0));
    vline(Offscreen,a+n,b,SCREEN_H-b,0);
    vline(Offscreen,a+(n<<1),b,SCREEN_H-b,0);
    hline(Offscreen,a,b+m,SCREEN_W-a,0);
    hline(Offscreen,a,b+(m<<1),SCREEN_W-a,0);
    for (int i=0; i<3; i++)
      for (int j=0; j<3; j++)
        switch (table[i][j])
        {
            case EMPTY: break;
            case X: line(Offscreen,a + n*i + OFFSET, b + m*j + OFFSET, a + n*i + n - OFFSET, m + m*j + b - OFFSET,0);
                    line(Offscreen,a + n*i + OFFSET, m + m*j + b - OFFSET, a + n*i + n - OFFSET, b + m*j + OFFSET,0);
                    break;
            case O: circle(Offscreen, i*n + a + (n>>1), j*m + b + (m>>1), ((m+n)>>2) - OFFSET, 0);
        }  
    alfont_set_font_size(MyFont,36);
    alfont_textout(Offscreen,MyFont,"Windowz Login",(SCREEN_W>>2),10,makecol(255,0,0));
    alfont_set_font_size(MyFont,30);
    alfont_textout(Offscreen,MyFont,"Defeat the game to access sensitive",50,50,makecol(255,0,0));
    alfont_textout(Offscreen,MyFont,"security informations..",(SCREEN_W>>3),90,makecol(255,0,0));    
}

void CTableGame :: UserAction()
{                             
    if (mouse_b & 1)   // left click
    {
        bool bFound = false;
        int x = 0;
        int y = 0;
        int n = (SCREEN_W - (SCREEN_W>>1)) / 3;
        int m = (SCREEN_H - (SCREEN_H>>1)) / 3;
        int a = SCREEN_W >> 2;
        int b = SCREEN_H >> 2;
        while ( (!bFound) && (y<3) )
        {
            while ( (!bFound) && (x<3) )
            {
                if ( (mouse_x >= a + x*n) && (mouse_x <= a + x*n+n) &&
                     (mouse_y >= b + y*m) && (mouse_y <= b + y*m+m) )
                {
                    bFound = true;
                    if (table[x][y] == EMPTY)
                    {
                        table[x][y] = X;
                        bComputersTurn = true;
                    }    
                }    
                else
                {
                    x++;
                }                    
            }                
            x = 0;
            y++;               
        }           
        // check for victory condition
        if ( ( (table[0][0] == X) && (table[0][1] == X) && (table[0][2] == X) ) ||
             ( (table[1][0] == X) && (table[1][1] == X) && (table[1][2] == X) ) ||
             ( (table[2][0] == X) && (table[2][1] == X) && (table[2][2] == X) ) ||
             ( (table[0][0] == X) && (table[1][0] == X) && (table[2][0] == X) ) ||
             ( (table[0][1] == X) && (table[1][1] == X) && (table[2][1] == X) ) ||
             ( (table[0][2] == X) && (table[1][2] == X) && (table[2][2] == X) ) ||
             ( (table[0][0] == X) && (table[1][1] == X) && (table[2][2] == X) ) ||
             ( (table[2][0] == X) && (table[1][1] == X) && (table[0][2] == X) ) )
        {
           Victory = true;
        } 
        else if (bComputersTurn)
        {
            bComputersTurn = false;
            x = 0;
            y = 0;
            bFound = false;
            while ( (!bFound) && (y < 3) )
            {
                while ( (!bFound) && (x < 3) )
                {    
                    if (table[x][y] == EMPTY)
                    {
                        table[x][y] = O;
                        bFound = true;
                    }
                    else x++;    
                }
                x = 0;
                y++;
            }
            if (!bFound)   // nobody won
            {
                for (x=0; x<3; x++)
                  for (y=0; y<3; y++)
                    table[x][y] = EMPTY;
            }    
        }       
    }    
}        

bool CTableGame :: XO()
{
    if (key[KEY_ESC]) Victory = true;
    DrawTable();
    UserAction();    
    return Victory;
}    

// ------------- BOOK READING -------------

CBookReader :: CBookReader(int interval)
{
    CountDown = interval;
    iInternalClock = 0;
}    

bool CBookReader :: ReadTheBook(volatile int &mtime)
{
    if (iInternalClock != mtime)
    {
        CountDown--;
        iInternalClock = mtime;
    }    
    if (CountDown <= 0)
    {
        Player->x -= SQUARE_WIDTH;
        Player->y += SQUARE_HEIGHT;
        return false;
    }    
    else
    {
        clear_bitmap(Offscreen);
        alfont_set_font_size(MyFont,36);
        alfont_textout(Offscreen,MyFont,"END USER LICENCE AGREEMENT",50,20,makecol(255,0,0));
        alfont_set_font_size(MyFont,20);
        alfont_textout(Offscreen,MyFont,"You must agree with this licence if you want to",20,65,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"use the following product of Macro$oft!!!",20,90,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"Failure to do so shall result in your death!",20,115,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"It is written that a hero shall come, from a far away",20,165,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"galaxy. The hero shall play Hell's game with the Devil",20,190,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"himself, and this saint shall win, and the gates of Hell",20,215,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"shall open. This shall be the begining of the end for",20,240,makecol(255,0,0));
        alfont_textout(Offscreen,MyFont,"the Devil's rule upon our world.",20,265,makecol(255,0,0));
        return true;
    }    
}    

// ------------ STARTUP SCREEN ------------------

CMainMenu :: CMainMenu(int interval)
{
    iState = 0;
    iInternalClock = 0;
    iCounter = interval;
    SplashScreen = load_bitmap(get_config_string("Globals","Startup","title.pcx"),NULL);    
    iMouseClickCounter = 5;
    bExplained = false;
    bDisplayExitMessage = false;
}

CMainMenu :: ~CMainMenu()
{
    destroy_bitmap(SplashScreen);    
}    
    
/* RESULTS:
    0 - do nothing
    1 - start game
    -1 - quit game
*/    
int CMainMenu :: MenuResult(volatile int &mtime)
{
    int x = (SCREEN_W >> 1) - 100;
    int y = 100;
    int w = 200;
    int h = 50;    
    switch (iState)
    {         
         case 0: if (iInternalClock != mtime)
                 {
                   iCounter--;
                   iInternalClock = mtime;
                 }   
                 if (!bExplained)
                 {
                     if (iCounter <= 0)
                     {
                         bExplained = true;
                         iCounter = 30;
                     }    
                     else
                     {
                        clear_bitmap(Offscreen);
                        stretch_sprite(Offscreen,SplashScreen,0,0,SCREEN_W,SCREEN_H);
                     }    
                 }   
                 else
                 {
                     if (key[KEY_SPACE] || key[KEY_ENTER] || (iCounter <= 0) || (mouse_b & 1))
                     {
                         iState = 1;
                     }    
                     else
                     {
                         clear_bitmap(Offscreen);                         
                         alfont_set_font_size(MyFont,20);
                         alfont_textout(Offscreen,MyFont,"The year is 17.538 and the Milky Way is ruled by a",20,90,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"ruthless corporation named Macro$oft. This corporation",20,115,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"uses various vicked techniques to keep the population",20,140,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"enslaved, while expanding the borders of its empire.",20,165,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"YOU are a member of a small ghost-like race located at",20,190,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"the edge of the galaxy, and your planet has just came",20,215,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"under attack by Macro$oft products and security.",20,240,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"updates. The Council of Elders has selected you to enter",20,265,makecol(255,0,0));                         
                         alfont_textout(Offscreen,MyFont,"your solar system's Macro$oft Localization Center and",20,290,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"make it incapable of working, i.e. kill its Big Boss.",20,315,makecol(255,0,0));
                         alfont_textout(Offscreen,MyFont,"Good luck!",20,355,makecol(255,0,0));                         
                     }    
                 }     
                 return 0;
         case 1: clear_bitmap(Offscreen);        
                 if (bDisplayExitMessage) textout(Offscreen,font,"You must restart the game now!",10,10,makecol(255,0,0));          
                 rectfill(Offscreen,x,y,x+w,y+h,(mouse_x >= x) && (mouse_x <= x+w) &&
                                                (mouse_y >= y) && (mouse_y <= y+h) ? 
                                                makecol(255,255,128) : makecol(128,255,255));
                 rectfill(Offscreen,x,y+100,x+w,y+100+h,(mouse_x >= x) && (mouse_x <= x+w) &&
                                                        (mouse_y >= y+100) && (mouse_y <= y+h+100) ? 
                                                        makecol(255,255,128) : makecol(128,255,255));
                 rectfill(Offscreen,x,y+200,x+w,y+200+h,(mouse_x >= x) && (mouse_x <= x+w) &&
                                                        (mouse_y >= y+200) && (mouse_y <= y+h+200) ? 
                                                        makecol(255,255,128) : makecol(128,255,255));
                 rect(Offscreen,x,y,x+w,y+h,0);
                 rect(Offscreen,x,y+100,x+w,y+100+h,0);
                 rect(Offscreen,x,y+200,x+w,y+200+h,0);
                 alfont_set_font_size(MyFont,22);
                 alfont_textout(Offscreen,MyFont,"Start Game",x+15,y+15,0);
                 alfont_textout(Offscreen,MyFont,"Options",x+33,y+115,0);
                 alfont_textout(Offscreen,MyFont,"Exit Game",x+18,y+215,0);
                 if (mouse_b & 1)
                 {
                     if (iMouseClickCounter < 5)
                     {
                         iMouseClickCounter++;
                         return 0;
                     }    
                     else
                     {
                       iMouseClickCounter = 0;            
                       if ((mouse_x >= x) && (mouse_x <= x+w) &&  // button 1
                           (mouse_y >= y) && (mouse_y <= y+h))
                       {
                           return 1;
                       }
                       else if ((mouse_x >= x) && (mouse_x <= x+w) &&  // button 2
                                (mouse_y >= y+100) && (mouse_y <= y+h+100))
                       {
                           iState = 2;
                           return 0;
                       }
                       else if ((mouse_x >= x) && (mouse_x <= x+w) &&  // button 3
                                (mouse_y >= y+200) && (mouse_y <= y+h+200))    
                       {
                           return -1;
                       }    
                   } 
                 }                      
                 return 0;                         
         case 2: clear_bitmap(Offscreen);   
                 if (bDisplayExitMessage) textout(Offscreen,font,"You must restart the game now!",10,10,makecol(255,0,0));
                 rectfill(Offscreen,x,y,x+w,y+h,(mouse_x >= x) && (mouse_x <= x+w) &&
                                                (mouse_y >= y) && (mouse_y <= y+h) ? 
                                                makecol(255,255,128) : makecol(128,255,255));
                 rectfill(Offscreen,x,y+100,x+w,y+100+h,(mouse_x >= x) && (mouse_x <= x+w) &&
                                                        (mouse_y >= y+100) && (mouse_y <= y+h+100) ? 
                                                        makecol(255,255,128) : makecol(128,255,255));
                 rectfill(Offscreen,x,y+200,x+w,y+200+h,(mouse_x >= x) && (mouse_x <= x+w) &&
                                                        (mouse_y >= y+200) && (mouse_y <= y+h+200) ? 
                                                        makecol(255,255,128) : makecol(128,255,255));
                 rect(Offscreen,x,y,x+w,y+h,0);
                 rect(Offscreen,x,y+100,x+w,y+100+h,0);
                 rect(Offscreen,x,y+200,x+w,y+200+h,0);
                 alfont_set_font_size(MyFont,22);
                 if (gfx == GFX_AUTODETECT_WINDOWED)
                 {
                    alfont_textout(Offscreen,MyFont,"Fullscreen",x+15,y+15,0);
                 }    
                 else
                 {
                     alfont_textout(Offscreen,MyFont,"Windowed",x+15,y+15,0);
                 }    
                 if (bPlayMusic)
                 {
                     alfont_textout(Offscreen,MyFont,"Music off",x+33,y+115,0);
                 }
                 else
                 {
                     alfont_textout(Offscreen,MyFont,"Music on",x+33,y+115,0);
                 }                         
                 alfont_textout(Offscreen,MyFont,"Back",x+18,y+215,0);
                 if (mouse_b & 1)
                 {
                     if (iMouseClickCounter < 4)
                     {
                         iMouseClickCounter++;
                         return 0;
                     }    
                     else
                     {
                       iMouseClickCounter = 0;  
                       if ((mouse_x >= x) && (mouse_x <= x+w) &&  // button 1
                           (mouse_y >= y) && (mouse_y <= y+h))
                       {
                           set_config_int("Globals","Windowed",gfx==GFX_AUTODETECT_WINDOWED ?
                                                               0 : 1);
                           bDisplayExitMessage = true;
                           return 0;
                       }
                       else if ((mouse_x >= x) && (mouse_x <= x+w) &&  // button 2
                                (mouse_y >= y+100) && (mouse_y <= y+h+100))
                       {
                           bPlayMusic = !bPlayMusic;
                           return 0;
                       }
                       else if ((mouse_x >= x) && (mouse_x <= x+w) &&  // button 3
                                (mouse_y >= y+200) && (mouse_y <= y+h+200))    
                       {
                           iState = 1;
                           return 0;
                       }    
                   }    
                 }
                 return 0; 
    }        
    return 0;
}    
