#include <allegro.h>
#include <alfont.h>
#include <math.h>
#include <string>
#include <sstream>
#include <list>
#include "basic.h"
#include "units.h"
#include "map.h"
#include "main.h"
#include "globals.h"
#include "controls.h"                              

using namespace std;

// global variables
static BITMAP* Pictures[MAX_UNIT_PICTURES];
static string MonsterTalk;
static int MonsterTalkCounter;
static SAMPLE* UnitSounds[UNIT_SOUND_COUNT];
int SampleVolume = 255;

bool LoadUnitPictures()
{
    Pictures[PLAYER_PICTURE_1] = load_bitmap(get_config_string("Player","Pic1","player1.pcx"),NULL);
    Pictures[PLAYER_PICTURE_2] = load_bitmap(get_config_string("Player","Pic2","player2.pcx"),NULL);
    Pictures[PLAYER_PICTURE_3] = load_bitmap(get_config_string("Player","Pic3","player3.pcx"),NULL);
    Pictures[CIVIL_PICTURE_1] = load_bitmap(get_config_string("Monsters","Civil1","civil1.pcx"),NULL);
    Pictures[CIVIL_PICTURE_2] = load_bitmap(get_config_string("Monsters","Civil2","civil2.pcx"),NULL);
    Pictures[CIVIL_PICTURE_3] = load_bitmap(get_config_string("Monsters","Civil3","civil3.pcx"),NULL);
    Pictures[SOLDIER_PICTURE_1] = load_bitmap(get_config_string("Monsters","Soldier1","soldier1.pcx"),NULL);
    Pictures[SOLDIER_PICTURE_2] = load_bitmap(get_config_string("Monsters","Soldier2","soldier2.pcx"),NULL);
    Pictures[SOLDIER_PICTURE_3] = load_bitmap(get_config_string("Monsters","Soldier3","soldier3.pcx"),NULL);
    Pictures[TROLL_PICTURE_1] = load_bitmap(get_config_string("Monsters","Troll1","troll1.pcx"),NULL);
    Pictures[TROLL_PICTURE_2] = load_bitmap(get_config_string("Monsters","Troll2","troll2.pcx"),NULL);
    Pictures[TROLL_PICTURE_3] = load_bitmap(get_config_string("Monsters","Troll3","troll3.pcx"),NULL);
    Pictures[BULLET_PICTURE_1] = load_bitmap(get_config_string("Bullet","Pic1","bullet1.pcx"),NULL);
    Pictures[BULLET_PICTURE_2] = load_bitmap(get_config_string("Bullet","Pic2","bullet2.pcx"),NULL);
    Pictures[BULLET_PICTURE_3] = load_bitmap(get_config_string("Bullet","Pic3","bullet3.pcx"),NULL);
    Pictures[BLOOD_PICTURE] = load_bitmap(get_config_string("Monsters","Blood","blood.pcx"),NULL);
    for (int i=0; i<MAX_UNIT_PICTURES; i++)
    {
        if (Pictures[i] == NULL) return false;
    }    
    return true;
}
    
void DestroyUnitPictures()
{
    for (int i=0; i<MAX_UNIT_PICTURES; i++)
       destroy_bitmap(Pictures[i]);
}  

bool LoadUnitSounds()
{
    UnitSounds[PLAYER_SOUND] = load_sample(get_config_string("Sounds","PlayerShoot","player.wav"));
    UnitSounds[SOLDIER_SOUND] = load_sample(get_config_string("Sounds","SoldierShoot","soldier.wav"));
    UnitSounds[TROLL_SOUND] = load_sample(get_config_string("Sounds","Troll","troll.wav"));
}

void DestroyUnitSounds()
{
    for (int i=0; i<UNIT_SOUND_COUNT; i++)
       destroy_sample(UnitSounds[i]);
}          

void CreateMonsters()
{
    int soldcount = get_config_int("Monsters","Soldiers",0);
    int civcount = get_config_int("Monsters","Civilians",0);
    int s = 0, c = 0;
    ostringstream tstr;
    for (int x=0; x<Map->iWidth; x++)
    {
        for (int y=0; y<Map->iHeight; y++)
        {
            switch (Map->DataMap[x][y])
            {
                case 4: tstr.str("");   // CIVILIAN
                        c++;
                        tstr << "Civil_" << c;
                        new CMonster(c,x*SQUARE_WIDTH,y*SQUARE_HEIGHT>>1,
                                     CIVILIAN,get_config_int(tstr.str().c_str(),"HP",100));
                        break;
                case 6: tstr.str("");   // SOLDIER
                        s++;
                        tstr << "Monster_" << s;
                        new CMonster(s,x*SQUARE_WIDTH,y*SQUARE_HEIGHT>>1,
                                     SOLDIER,get_config_int(tstr.str().c_str(),"HP",300));
                        break;
                case 9: tstr.str("");   // TROLL                        
                        tstr << "Troll";
                        Troll = new CMonster(s+c,x*SQUARE_WIDTH,y*SQUARE_HEIGHT>>1,
                                            TROLL,get_config_int(tstr.str().c_str(),"HP",1000));
                        break;
            }    
        }    
    }    
}
    
void DestroyMonsters()
{
    list<CUnit*>::iterator i = Monsters.begin();
    while (i != Monsters.end())
    {
        i = Monsters.erase(i);        
    }    
}    

void DrawBullets()
{
    list<CUnit*>::iterator i = Bullets.begin();
    while (i != Bullets.end())
    {
        ((CBullet*)(*i))->DoDraw();
        i++;
    }
}

void MonsterAction()
{
    list<CUnit*>::iterator i = Monsters.begin();
    while (i != Monsters.end())
    {
        ((CMonster*)(*i))->DoAction();
        i++;
    }
}

void BulletAction()
{
    list<CUnit*>::iterator i = Bullets.begin();
    while (i != Bullets.end())
    {
        ((CBullet*)(*i))->DoAction();
        i++;
    }            
}

void DestroyBullets()
{
    list<CUnit*>::iterator i = Bullets.begin();
    while (i != Bullets.end())
    {
        i = Bullets.erase(i);
    }    
}                

// ------------------- CUnit ----------------------
CUnit :: CUnit()
{
    bAlive = true;
    HP = 0;
}    

// ------------------- CPlayer ----------------------

CPlayer :: CPlayer(int x, int y, int HP)        // constructor
{
    this->x = x;        // player coordinates
    this->y = y;
    this->HP = HP;     // hit points
    this->bAlive = true;   // is player alive
    this->iGoX = -1;   // don't go anywhere
    this->iGoY = -1;
    this->iFireRange = get_config_int("Player","FireRange",10);
    this->iMaxImageFrames = PLAYER_PICTURE_FRAMES;  // image frames
    this->iImageFrame = 0;    // current image frame
    this->ItsPicture = new BITMAP*[PLAYER_PICTURE_FRAMES];  // create array of bitmaps
    this->ItsPicture[0] = Pictures[PLAYER_PICTURE_1];   // set each frame
    this->ItsPicture[1] = Pictures[PLAYER_PICTURE_2];
    this->ItsPicture[2] = Pictures[PLAYER_PICTURE_3];
    this->w = ItsPicture[0]->w;     // get player width
    this->h = ItsPicture[0]->h;     // get player height
    this->iState = STOP;
    this->iDirection = NORTHWEST;
    this->iFireFrame = 0;
    this->Talker = NULL;    
    this->iPause = 0;
}
    
CPlayer :: ~CPlayer()   // destructor
{
    delete ItsPicture;
}

// returns the distance between two points
inline float Dist(int x1, int y1, int x2, int y2)
{
    int x = x2-x1;
    int y = y2-y1;
    return sqrt((float)(x*x + y*y));
}    

inline bool MoveUnit(int &x, int &y, int &w, int &h, int &iGoX, int &iGoY, int &iDirection)
{
    int tx = x;   // try x
    int ty = y;   // try y
    
    if ( iGoX > x )
    {
       if (iGoY > y)
       {
            iDirection = SOUTHWEST;
            ty += PLAYER_STEP;
       }
       else  // iGoY <= y
       {
            iDirection = NORTHWEST;         
            ty -= PLAYER_STEP;     
       }        
       tx += PLAYER_STEP;
    }                                          
    else   // iGoX <= x
    {
         if (iGoY > y)
         {
            iDirection = SOUTHEAST;
            ty += PLAYER_STEP;
         }
         else  // iGoY <= y
         {
             iDirection = NORTHEAST;
             ty -= PLAYER_STEP;                     
         }        
         tx -= PLAYER_STEP;
    }
         
    if (Map->Collision(tx,ty,w,h))      // something else is on that coordinate
    {
        return false;
    }
    else
    {
        x = tx;
        y = ty;
        return true;
    }                              
}    

inline void PlayerFireTowards(int x, int y, int iGoX, int iGoY, int &iFireFrame, CUnit* Owner)
{
    if (iFireFrame == FIRING_INTERVAL)
    {
        // the bullet will add itself to the list
        new CBullet(x+10,y+10,iGoX+10,iGoY+10,BULLET_SPEED,Owner);
        iFireFrame = 0;        
        play_sample(UnitSounds[PLAYER_SOUND],SampleVolume,0,1000,0);
    }
    else
    {
        iFireFrame++;       
    }                                            
}   

void DrawTalking()
{
    if (MonsterTalkCounter == TALK_LIMIT)
    {
        MonsterTalk = "";
    }  
    else if (MonsterTalk != "")
    {
        MonsterTalkCounter++;
        alfont_set_font_size(MyFont, 14);
        alfont_textout(Offscreen,MyFont,MonsterTalk.c_str(),10,Menu->h + 10,makecol(255,0,0));
    }      
}     

inline void TalkTo(int x, int y, int iGoX, int iGoY)
{        
    MonsterTalkCounter = 0;
    MonsterTalk = ((CMonster*)Player->Talker)->Taunt();    
    if (((CMonster*)Player->Talker)->iTextCounter < TEXT_COUNT-1) ((CMonster*)Player->Talker)->iTextCounter++;
}    
    
void CPlayer :: DoAction()
{    
    switch (iState)
    {
        case STOP: /* do nothing? */ break;
        case MOVE: if ( (iGoX == -1) || (iGoY == -1) )
                   {
                     iState = STOP;
                   }
                   else if ( (iGoX >= x - PLAYER_STEP) && (iGoX <= x + PLAYER_STEP) && 
                             (iGoY >= y - PLAYER_STEP) && (iGoY <= y + PLAYER_STEP) )
                   {
                       iState = STOP;  // we have reached the designation 
                   }
                   else if (!MoveUnit(x,y,w,h,iGoX,iGoY,iDirection)) iState = STOP;   
                   break;
        case ATTACK: if (Dist(x,y,iGoX,iGoY) > iFireRange)
                        iState = MOVE;
                     else
                        PlayerFireTowards(x,y,iGoX,iGoY,iFireFrame,this);
                     break;
        case TALK: if (Dist(x,y,iGoX,iGoY) > iFireRange)
                      iState = MOVE;
                    else     
                      if (iPause == 3)  // force some delay when right-clicking
                      {                  
                      TalkTo(x,y,iGoX,iGoY); 
                      iState = STOP;                         
                      iPause = 0;
                      }    
                      else iPause++;
                    break;
    }    
}
    
void CPlayer :: DoDraw()
{    
    int iAngle;
    switch (iDirection)             // FIX THIS!!!
    {
        case NORTHWEST: iAngle = 0;  break;   // do nothing
        case NORTHEAST: iAngle = 192; break;  // left counter-clockwise
        case SOUTHWEST: iAngle = 64; break;   // right clockwise
        case SOUTHEAST: iAngle = 128; break;  // flip
    }    
    iAngle = 0;    // OVERRIDE!
    rotate_sprite(Offscreen,ItsPicture[iImageFrame],x-Map->iGlobalX,y-Map->iGlobalY-(SQUARE_WIDTH>>1),itofix(iAngle));    
    if (iState != STOP) iImageFrame++;   // only move when you MOVE
    if (iImageFrame >= iMaxImageFrames) iImageFrame = 0;
}    

// ------------------- CMonster ----------------------

CMonster :: CMonster(int ID, int x, int y, int iType, int HP)
{                 
    this->x = x;
    this->y = y;
    this->iType = iType;
    this->HP = HP;
    this->bAlive = true;
    this->iTextCounter = 0;
    this->ID = ID;    
    this->iFireRange = get_config_int("Monsters","FireRange",10);
    ostringstream idtmp;
    if (iType == CIVILIAN)
    {
        idtmp.str("");
        idtmp << "Civil_" << ID;
    }
    else if (iType == SOLDIER)
    {
        idtmp.str("");
        idtmp << "Monster_" << ID;
    }
    else
    {
        idtmp.str("");
        idtmp << "Troll";
    }            
    this->iType = get_config_int(idtmp.str().c_str(),"Type",0);
    this->bCivilian = iType == CIVILIAN;
    this->szText[0] = get_config_string(idtmp.str().c_str(),"Text1","Hello");
    this->szText[1] = get_config_string(idtmp.str().c_str(),"Text2","What's your name?");
    this->szText[2] = get_config_string(idtmp.str().c_str(),"Text3","Are you a friend?");
    
    int tf = 0, tp1 = 0, tp2 = 0, tp3 = 0;
    switch (iType)
    {
        case CIVILIAN: tf = CIVIL_PICTURE_FRAMES; 
                       tp1 = CIVIL_PICTURE_1;
                       tp2 = CIVIL_PICTURE_2;
                       tp3 = CIVIL_PICTURE_3;
                       break;
        case SOLDIER: tf = SOLDIER_PICTURE_FRAMES; 
                      tp1 = SOLDIER_PICTURE_1;
                      tp2 = SOLDIER_PICTURE_2;
                      tp3 = SOLDIER_PICTURE_3;
                      break;
        case TROLL: tf = TROLL_PICTURE_FRAMES; 
                    tp1 = TROLL_PICTURE_1;
                    tp2 = TROLL_PICTURE_2;
                    tp3 = TROLL_PICTURE_3;
                    break;
    }    
    this->ItsPicture = new BITMAP*[tf];  // create array of bitmaps
    this->ItsPicture[0] = Pictures[tp1];   // set each frame
    this->ItsPicture[1] = Pictures[tp2];
    this->ItsPicture[2] = Pictures[tp3];
    this->iMaxImageFrames = tf;
    this->iImageFrame = 0;
    this->w = ItsPicture[0]->w;     // get monster width
    this->h = ItsPicture[0]->h;     // get monster height
    this->iState = STOP;
    this->iDirection = NORTHWEST;
    this->iFireFrame = 0;
    this->iState = STOP;    
    // automaticaly add this object to the appropriate list
    Monsters.push_back(this);
}
    
CMonster :: ~CMonster()
{
    delete ItsPicture;
}
    
void CMonster :: SetText(int index, string txt)
{           
    if ( (index >= 0) && (index <= TEXT_COUNT) )
       this->szText[index] = txt;
}
    
string CMonster :: Taunt()
{    
    return szText[iTextCounter];
}

inline void MonsterFireTowards(int x, int y, int iGoX, int iGoY, int &iFireFrame, CUnit* Owner)
{
    if (iFireFrame == FIRING_INTERVAL+2)  // shoot slower than the player
    {
        // the bullet will add itself to the list
        new CBullet(x+10,y+10,iGoX+10,iGoY+10,BULLET_SPEED,Owner);
        iFireFrame = 0;        
        if (Owner != Troll)
          play_sample(UnitSounds[SOLDIER_SOUND],SampleVolume,0,1000,0);
        else
          play_sample(UnitSounds[TROLL_SOUND],SampleVolume,0,1000,0);
    }
    else
    {
        iFireFrame++;       
    }                                           
}   
    
void CMonster :: DoAction()
{        
    if (HP <= 0) 
    {
        bAlive = false;  // let's check if our unit is dead..
        Player->iGoX = -1;
        Player->iGoY = -1;
    }    
    else  // it's alive!!
    {
        if ( (iType == CIVILIAN) )
        {            
            if (this->HP < DEFAULT_CIVIL_HP)
            {
                // let's try to run away
                iState = MOVE;
                MoveUnit(x,y,w,h,Player->y,Player->x,iDirection);        
            }
        }
        else if (this->iType == SOLDIER)
        {
          if ( (this->HP < DEFAULT_SOLDIER_HP) ||  // wounded, attack all the time!
               (bBookRead) )    // player has read the book, he must be destroyed! 
          {
            if (Dist(x,y,Player->x,Player->y) <= iFireRange)
            {
                iState = ATTACK;
                MonsterFireTowards(x,y,Player->x,Player->y,iFireFrame,this);
            }
            else
            {
                iState = STOP;
            }        
           }    
           else
           {
               // the soldier may fire at random moments even if not wounded
               if ((rand() % 100 < 20) && (Dist(x,y,Player->x,Player->y) <= iFireRange))  
               {
                   iState = ATTACK;
                   MonsterFireTowards(x,y,Player->x,Player->y,iFireFrame,this);
               }   
               else
               {
                   iState = STOP;
               }     
           }    
        }        
        else if (this->iType == TROLL)
        {          
            if (Dist(x,y,Player->x,Player->y) <= iFireRange)
            {
                iState = ATTACK;
                MonsterFireTowards(x,y,Player->x,Player->y,iFireFrame,this);
            }
            else
            {
                iState = STOP;
            }                   
        }
    }    
}
    
void CMonster :: DoDraw()
{
    if (!bAlive) 
    {
        draw_sprite(Offscreen,Pictures[BLOOD_PICTURE],x-Map->iGlobalX,y-Map->iGlobalY);
        return;
    }    
    int iAngle;
    switch (iDirection)
    {
        case NORTHWEST: iAngle = 0;  break;   // do nothing
        case NORTHEAST: iAngle = 192; break;  // left counter-clockwise
        case SOUTHWEST: iAngle = 64; break;   // right clockwise
        case SOUTHEAST: iAngle = 128; break;  // flip
    }    
    iAngle = 0; // OVERRIDE
    rotate_sprite(Offscreen,ItsPicture[iImageFrame],x-Map->iGlobalX,y-Map->iGlobalY,iAngle);    
    // textprintf(Offscreen,font,x-Map->iGlobalX,y-Map->iGlobalY,255,"%d",HP);
    if (iState != STOP) iImageFrame++;
    if (iImageFrame >= iMaxImageFrames) iImageFrame = 0;    
}    

// ------------------- CBullet ----------------------

CBullet :: CBullet(int x, int y, int gx, int gy, int speed, CUnit* Owner)
{
    this->x = x;
    this->y = y;
    this->iGoalX = gx;
    this->iGoalY = gy;
    this->iSpeed = speed;
    this->Owner = Owner;
    this->HP = 10000000;
    this->bAlive = true;
    this->iImageFrame = 0;
    this->iMaxImageFrames = BULLET_PICTURE_FRAMES;
    this->ItsPicture = new BITMAP*[BULLET_PICTURE_FRAMES];
    this->ItsPicture[0] = Pictures[BULLET_PICTURE_1];
    this->ItsPicture[1] = Pictures[BULLET_PICTURE_2];
    this->w = ItsPicture[0]->w;     
    this->h = ItsPicture[0]->h;
    this->iFireFrame = 0;
    this->iState = MOVE;     // duh    
    // add this bullet to the appropriate list
    Bullets.push_back(this);
}
    
CBullet :: ~CBullet()
{
    delete ItsPicture;
}
    
void CBullet :: DoAction()
{       
   double alfa = 0.0;
   int sign = 1;
   double step = (double)iSpeed;
   int ERROR = iSpeed + 1;
   if ( ((x > iGoalX) & (y > iGoalY)) || ((x <= iGoalX) & (y <= iGoalY)) )
   {
      sign = 1;     //  north-west and south-east
   }
   else
   {
      sign = -1;    // south-west and north-east
   }
   if (iGoalX != x)    // division by zero
   {
      alfa = sign * atan(((double)(iGoalY-y))/((double)(iGoalX-x)));
   }
   if ( !((x > iGoalX-ERROR) & (x < iGoalX+ERROR)) )
   {
      if (x > iGoalX)
      {
         x -= (int)(step * cos(alfa));
      }
      else
      {
         x += (int)(step * cos(alfa));
      }
   }

   if ( !((y > iGoalY-ERROR) & (y < iGoalY+ERROR)) )
   {

      if (y > iGoalY)
      {
         y -= (int)(step * sin(alfa));
      }
      else
      {
         y += (int)(step * sin(alfa));
      }

   }
   
   if ( (x > iGoalX-ERROR) && (x < iGoalX+ERROR) &&
        (y > iGoalY-ERROR) && (y < iGoalY+ERROR) )
   {
       this->bAlive = false;   // bullet is destroyed       
   }    
   else if ( Map->Collision(x,y,w,h) )
   {
       this->bAlive = false;    // hit into a wall
   }   
   // let's see if the bullet has hit something   
   int X = x+(w>>1);
   int Y = y+(h>>1);
   if ( (Owner != Player) &&
        (X >= Player->x) && (X <= Player->x + Player->w) &&
        (Y >= Player->y) && (Y <= Player->y + Player->h) )
   {
       if (bAlive)   // a bullet should take down HP only once ;)
       {
          this->bAlive = false;
          Player->HP -= BULLET_STRENGTH;
       }    
   }
   else
   {    
      list<CUnit*>::iterator i = Monsters.begin(); 
      while ( (i != Monsters.end()) && (bAlive) )
      {
          if ( (Owner != *i) &&
               (X >= (*i)->x) && (X <= (*i)->x + (*i)->w) &&
               (Y >= (*i)->y) && (Y <= (*i)->y + (*i)->h) )
          {
              if (bAlive)
              {
                  (*i)->HP -= BULLET_STRENGTH;
                  bAlive = false;
              }                                
          }
          else
          {
              i++;
          }        
      }    
   }    
}
    
void CBullet :: DoDraw()
{
    if (Owner != Player)
    {
       draw_sprite(Offscreen,Pictures[BULLET_PICTURE_3],x-Map->iGlobalX,y-Map->iGlobalY);
       return;
    }    
    int iAngle;
    if (iGoalX != x)    // division by zero
    {
      iAngle = (int)(255*atan(((double)(iGoalY-y))/((double)(iGoalX-x))));
    }
    else
    {                             
       iAngle = 0;
    }       
    rotate_sprite(Offscreen,ItsPicture[iImageFrame],x-Map->iGlobalX,y-Map->iGlobalY,itofix(iAngle));
    if ((x+y) % 2 == 0) return;  // animate only sometimes
    if (iState != STOP) iImageFrame++;
    if (iImageFrame >= iMaxImageFrames) iImageFrame = 0;
}    
