/*
********************************************************
*                                                      *
*             MOVING OBJECTS (UNITS)                   *
*                                                      *
********************************************************

This is where the three basic units will be defined: the player,
monsters and bullets. This includes the implementation of their
actions and drawing on the screen

Author: Gajo Csaba
Date: 25 Feb, 2005
Version: 0.5
*/

#ifndef _units_h
#define _units_h

#include <string>
#include "basic.h"

#define PLAYER_MAX_HP 2000
#define DEFAULT_CIVIL_HP 100
#define DEFAULT_SOLDIER_HP 300
#define ENERGY_CHARGE 100  // how much energy will the player get from a medikit

#define CIVILIAN 0
#define SOLDIER 1
#define TROLL 2

#define TEXT_COUNT 3     // how much text can a unit speak
#define UNIT_SOUND_COUNT 3    // how much samples are used by the units

#define MAX_UNIT_PICTURES 16      // how many unit pictures we have
#define PLAYER_PICTURE_1 0
#define PLAYER_PICTURE_2 1
#define PLAYER_PICTURE_3 2
#define CIVIL_PICTURE_1 3
#define CIVIL_PICTURE_2 4
#define CIVIL_PICTURE_3 5
#define SOLDIER_PICTURE_1 6
#define SOLDIER_PICTURE_2 7
#define SOLDIER_PICTURE_3 8
#define TROLL_PICTURE_1 9
#define TROLL_PICTURE_2 10
#define TROLL_PICTURE_3 11
#define BULLET_PICTURE_1 12
#define BULLET_PICTURE_2 13
#define BULLET_PICTURE_3 14
#define BLOOD_PICTURE 15

#define PLAYER_SOUND 0
#define SOLDIER_SOUND 1
#define TROLL_SOUND 2

#define PLAYER_PICTURE_FRAMES 3      // how many frames a unit has
#define CIVIL_PICTURE_FRAMES 3
#define SOLDIER_PICTURE_FRAMES 3
#define TROLL_PICTURE_FRAMES 3
#define BULLET_PICTURE_FRAMES 2

#define STOP 0     // unit states
#define MOVE 1
#define ATTACK 2
#define TALK 3

#define NORTHWEST 0  // unit directions
#define NORTHEAST 1
#define SOUTHWEST 2
#define SOUTHEAST 3

#define PLAYER_STEP 3   // how much can a player move
#define MONSTER_STEP 1
#define TROLL_STEP 3

#define BULLET_SPEED 10
#define FIRING_INTERVAL 10  // fire each 10 frames
#define BULLET_STRENGTH 50

#define TALK_LIMIT 50

extern int SampleVolume;

class BITMAP;

class CUnit : public CObject    // abstract class
{
    public:
        bool bAlive;
        int HP;        
        CUnit();
        int iState;
        int iDirection;
        int iFireFrame;
};

class CPlayer : public CUnit
{
    private:        
        int iFireRange;        
    public:
        int iPause;
        int iGoX,iGoY;
        std::string szName;   // player name
        CUnit* Talker;
        bool bHasDisk;      // has the player aquired the disk
        bool iKilled;       // how many units the player has killed
        CPlayer(int x, int y, int HP);
        ~CPlayer();
        void DoAction();
        void DoDraw();
};
    
class CMonster : public CUnit
{
    private:
//        int iTextCounter;      // how much text has the monster said
        int ID;
        int iFireRange;
    public:        
        int iTextCounter;
        std::string szText[TEXT_COUNT];  // the unit's speech
        bool bCivilian;                  // is the unit a civilian
        int iType;                       // type of unit
        CMonster(int ID, int x, int y, int iType, int HP);
        ~CMonster();
        void SetText(int index, std::string txt);
        std::string Taunt();     // speak with the monster
        void DoAction();
        void DoDraw();
        
};
    
class CBullet : public CUnit
{
    private:
        CUnit* Owner;
    public:
        int iGoalX;     // coordinates of the target
        int iGoalY;
        int iSpeed;
        CBullet(int x, int y, int gx, int gy, int speed, CUnit* Owner);
        ~CBullet();
        void DoAction();
        void DoDraw();
}; 

bool LoadUnitPictures();   
void DestroyUnitPictures();
bool LoadUnitSounds();
void DestroyUnitSounds();
void CreateMonsters();
void DestroyMonsters();
void DrawBullets();
void MonsterAction();
void BulletAction();
void DestroyBullets();
void DrawTalking();

#endif
